package wvpaint;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.awt.font.*;

/**
 * the interface that handles Rectangles, Ovals, text and lines
 *  @author danieltv and isaaclw
 */
public interface PaintObject {

    public final static int SIZE = 20;
    //rectangle oval, text

    /**
     * Gets the location of the object in the case of a move, or remove
     * @return a RectangluarShape around the object
     */
    public RectangularShape getObject();

    /**
     * Re-draws the individual object
     * @param g Graphics2D
     */
    public void draw(Graphics2D g);

    /**
     * Moves the object
     * @param x the x coordinate the object will be moved to
     * @param y the y coordinate the object will be moved to
     */
    public void move(double x, double y);

    /**
     * sets the color of the object
     * @param c the Color the object will be set to.
     */
    public void setColor(Color c);
}

/**
 * A Rectangle class
 */
class OurRect extends JComponent implements PaintObject {

    private Rectangle2D rect; // The visual Rectangle
    private Color clr; // The color for the Rectangle
    private boolean fill; // whether or not the rectangle is filled

    /**
     * The Constructor for a Rectangle
     * @param x The initial x coordinate
     * @param y The initial y coordinate
     * @param aColor The initial color of the box
     * @param aFill aFill = true, the shape will be filled
     */
    public OurRect(double x, double y, Color aColor, boolean aFill) {
        rect = new Rectangle2D.Double(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
        //save the variables
        clr = aColor;
        fill = aFill;
    }

    public void draw(Graphics2D g) {
        g.setColor(clr);
        if (fill) {
            g.fill(rect);
        } //checks if the rectangle is supposed to be filled, and fills it
        else {
            g.draw(rect);
        }
    }

    public void move(double x, double y) {
        rect.setFrame(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
    }

    public Rectangle2D getObject() {
        return rect;
    }

    public void setColor(Color c) {
        clr = c;
    }
}

/**
 * A Oval class
 */
class OurOval extends JComponent implements PaintObject {

    private Ellipse2D oval; // The visual Oval
    private Color clr; // The color for the Oval
    private boolean fill; // whether or not the oval is filled

    /**
     * The Constructor for a Oval
     * @param x The initial x coordinate
     * @param y The initial y coordinate
     * @param aColor The initial color of the circle
     * @param aFill aFill = true, the shape will be filled
     */
    public OurOval(double x, double y, Color aColor, boolean aFill) {
        oval = new Ellipse2D.Double(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
        clr = aColor;
        fill = aFill;
    }

    public void draw(Graphics2D g) {
        g.setColor(clr);
        if (fill) {
            g.fill(oval);
        } else {
            g.draw(oval);
        }
    }

    public void move(double x, double y) {
        oval.setFrame(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
    }

    public Ellipse2D getObject() {
        return oval;
    }

    public void setColor(Color c) {
        clr = c;
    }
}

/**
 * A Text Class
 */
class OurText extends JComponent implements PaintObject {

    private String text; // the text that appears
    private Color clr; //the color for the string
    private int x,  y; // The values this time have to be passed in because there
    // is no Object (like Ellipse2D) which holds that for us
    private Rectangle2D frame; //the frame of the rectangle

    /**
     * The Constructor for Text
     * @param txt The text that will be painted
     * @param ax The initial x coordinate
     * @param ay The initial y coordinate
     * @param aColor The color of the String
     */
    public OurText(String txt, double ax, double ay, Color aColor) {
        //save the variables
        text = txt;
        clr = aColor;
        x = (int) ax;
        y = (int) ay;
        //make a frame around the text
        frame = new Rectangle2D.Double(x - SIZE / 2 * 3, y - SIZE / 2, SIZE * 3, SIZE / 2);
    }

    public void draw(Graphics2D g) {
        g.setColor(clr);
        g.drawString(text, x - SIZE / 2 * 3, y);
    }

    public void move(double ax, double ay) {
        //save the variables
        x = (int) ax;
        y = (int) ay;
        //relocate the frame
        frame.setFrame(x - SIZE / 2 * 3, y - SIZE / 2, SIZE * 3, SIZE / 2);
    }

    public Rectangle2D getObject() {
        return frame;
    }

    public void setColor(Color c) {
        clr = c;
    }
}

/**
 * A Line class
 */
class OurLine extends JComponent implements PaintObject {

    private Line2D line; // the visual line
    private Rectangle2D frame; // The frame around the line
    private Color clr; // the Color of the line

    /**
     * Constructing the line
     * @param x the x coordinate of the click
     * @param y the y coordinate of the click
     * @param aColor
     */
    public OurLine(double x, double y, Color aColor) {
        // centers the line
        line = new Line2D.Double(x - SIZE / 2, y, x + SIZE / 2, y);
        // makes the frame
        frame = new Rectangle2D.Double(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
        // saves the color
        clr = aColor;
    }

    public void draw(Graphics2D g) {
        g.setColor(clr);
        g.draw(line);
    }

    public void move(double x, double y) {
        line.setLine(x - SIZE / 2, y, x + SIZE / 2, y);
        frame.setFrame(x - SIZE / 2, y - SIZE / 2, SIZE, SIZE);
    }

    public Rectangle2D getObject() {
        return frame;
    }

    public void setColor(Color c) {
        clr = c;
    }
}